import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;


/**
 * This class is to make an action to add the queue to the monitor.
 *
 * @author tjacopi
 *
 */
public class AddQueueAction extends MonitorFrameAction {

    public void actionPerformed(ActionEvent e) {
      MonitorFrame parent = getParentFrame( e.getSource() );
      doAddQueue(parent);
    }

    public void doAddQueue(MonitorFrame parent) {
      ReplInstanceMonitorPanel monitorPanel = parent.getMonitorPanel();
      int graphTypes = ReplInstanceMonitorPanel.ThroughputGraph;
      if (monitorPanel instanceof QApplyInstanceMonitorPanel) {          // This is a hack, but if qapply
        graphTypes = graphTypes | ReplInstanceMonitorPanel.LatencyGraph; // ..then we also want to monitor latency
      };

      Cursor savedCursor = parent.getCursor();
      parent.setCursor( Cursor.getPredefinedCursor(Cursor.WAIT_CURSOR) );
      Vector allQueues = monitorPanel.getAllQueues();
      parent.setCursor( savedCursor );
      QueueSelectionDialog selDialog = new QueueSelectionDialog(parent, "Add Queue", true, allQueues);
      selDialog.pack();
      centerWindow(selDialog, parent);
      selDialog.show();
      if (selDialog.wasOkPressed()) {
        Object[] queues = selDialog.getSelectedQueues();
        boolean repackNecessary = false;
        for (int i=0; i<queues.length; i++) {
          String queue = queues[i].toString();
          boolean repack = monitorPanel.addMonitoring( graphTypes, queue);
          if (repack) {
            repackNecessary = true;
          }
        };
        if (repackNecessary) {
          monitorPanel.doLayout();
          parent.pack();
        };
      };
    }


  protected static void centerWindow(Window w, Window parent) {
    Rectangle rParent = parent.getBounds();
    Dimension dimWindow = w.getSize();
    int x = rParent.x + (rParent.width/2) - (dimWindow.width/2);
    int y = rParent.y + (rParent.height/2) - (dimWindow.height/2);
    if (x <0)
      x=0;
    if (y <0)
      y=0;

    w.setLocation(x,y);
  }

}