import java.io.*;
import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import java.util.*;


/**
 * This class has some base functions that are used by all the monitors.
 *
 * @author tjacopi
 *
 */
public class MonitorBase  {

  protected static String aAlias = null;
  protected static String aSchema = null;
  protected static String aUid = null;
  protected static String aPwd = null;
  protected static boolean useProfile = true;

  protected static void init(String args[]) {
    try {
      UIManager.setLookAndFeel("com.sun.java.swing.plaf.windows.WindowsLookAndFeel");
    } catch (Exception e) {
      e.printStackTrace();
    }

    // Get the arguments the user passed in
    aAlias = parseArgsFor("-alias", args);
    aSchema = parseArgsFor("-schema", args);
    aUid = parseArgsFor("-uid", args);
    aPwd = parseArgsFor("-pwd", args);
    useProfile = !doesArgExist("-noprof", args);

    if (aSchema == null || aSchema.length() == 0 ){
      aSchema = "ASN";
    };

  }


  protected static void printParams() {
    System.out.println("  parameters:      ");
    System.out.println("     -alias  :  Required.  This is the alias where the replicaiton instance is located.");
    System.out.println("     -schema :  Optional, defaults to ASN.  The schema at the replication instance.");
    System.out.println("     -uid    :  Required.  The userid used to connect to the apply alias.");
    System.out.println("     -pwd    :  Required.  The password for the userid.");
    System.out.println("     -noprof :  Optional.  If specified, the profile will not be used (containing last used positions & queues)");
    System.out.println("                                               ");
  }

  protected static String parseArgsFor(String argName, String[] args) {
    String argValue = null;
    for (int i =0; i<args.length-1 && argValue == null; i++) {
      if (argName.equalsIgnoreCase(args[i]) ) {
        argValue = args[i+1];
      }
    }
    return argValue;
  }

  protected static boolean doesArgExist(String argName, String[] args) {
    boolean exist   = false;
    for (int i =0; i<args.length && !exist; i++) {
      exist = argName.equalsIgnoreCase(args[i]);
    }
    return exist;
  }

  protected static void centerWindow(Window w) {
    Dimension dimScreen = Toolkit.getDefaultToolkit().getScreenSize();
    Dimension dimWindow = w.getSize();
    int x = (dimScreen.width/2) - (dimWindow.width/2);
    int y = (dimScreen.height/2) - (dimWindow.height/2);
    w.setLocation(x,y);
  }

  protected static Vector formatString(String s) {
    Vector lines = new Vector();
    StringTokenizer st = new StringTokenizer(s, " ");
    String line = "   ";
    while (st.hasMoreTokens() ) {
      String str = st.nextToken();
      if (str.length() + line.length() < 80) {
        line = line + " " + str;
      } else {
        lines.add(line);
        line = "   " + str;
      }
    }
    lines.add(line);
    return lines;
  }


  public static void writeSavedData(Hashtable data) {
    try {
      FileOutputStream ostream = new FileOutputStream("replmon.dat");
      ObjectOutputStream p = new ObjectOutputStream(ostream);
      p.writeObject(data);
      p.flush();
      ostream.close();
    } catch (Exception e) {
      String txt = "Error writing saved data to file replmon.dat.\nJava error:";

      Vector lines = formatString(e.toString() );
      for (int j=0;j<lines.size();j++) {
        txt = txt + "\n" + lines.elementAt(j).toString();
      }
      e.printStackTrace();
      JOptionPane.showMessageDialog(null, txt, "Error", JOptionPane.ERROR_MESSAGE);
    }

  }

  public static Hashtable readSavedData() {
    Hashtable ht = null;
    try {
      FileInputStream istream = new FileInputStream("replmon.dat");
      ObjectInputStream p = new ObjectInputStream(istream);
      ht = (Hashtable) p.readObject();
      istream.close();
    } catch (Exception e) {
      System.out.println("Could not read saved position info in replmon.dat file");
    }

    if (ht == null) {
      ht = new Hashtable();
    };

    return ht;
  }

  protected static void addShutdownListener(MonitorFrame monitorFrame, boolean bExit) {
    if (bExit) {
      monitorFrame.addWindowListener(new WindowAdapter() {
	      public void windowClosing(WindowEvent e) {
           MonitorFrame mf = (MonitorFrame) e.getSource();
           closeFrameAndShutdown(mf, true);
         }
      });
    } else {
      monitorFrame.addWindowListener(new WindowAdapter() {
	      public void windowClosing(WindowEvent e) {
           MonitorFrame mf = (MonitorFrame) e.getSource();
           closeFrameAndShutdown(mf, false);
         }
      });
    }
  }

  public static void closeFrameAndShutdown(MonitorFrame monitorFrame, boolean bExit) {
    Hashtable ht = readSavedData();
    monitorFrame.saveInformation(ht);
    writeSavedData(ht);
    if (bExit) {
      System.exit(0);
    }
  }


}