import java.sql.*;
import java.awt.*;
import java.util.*;
import java.awt.datatransfer.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.event.*;
import javax.swing.border.*;


/**
 * The MonitorFrame is the JFrame that holds the MonitorTablePanel.  It provides support
 * for the menu items.
 *
 * @author tjacopi
 *
 */
public class MonitorFrame extends JFrame implements ActionListener, MenuListener {
  protected static final ImageIcon horizontalLayoutIcon = new ImageIcon("horizLayout.gif");
  protected static final ImageIcon verticalLayoutIcon = new ImageIcon("vertLayout.gif");

  protected ReplInstanceMonitorPanel monitorPanel = null;
  protected JMenuItem copyMI = null;
  protected JMenuItem refreshMI = null;
  protected JMenuItem exitMI = null;
  protected JMenuItem helpMI = null;
  protected JMenuItem startMI = null;
  protected JMenuItem stopMI = null;
  protected JMenuItem aboutMI = null;
  protected JMenuItem selectAllMI = null;
  protected JMenuItem trace1MinMI = null;
  protected JMenuItem trace10MinMI = null;
  protected JMenuItem trace1HrMI = null;
  protected JMenuItem trace1DayMI = null;
  protected JMenuItem traceAllMI = null;
  protected JMenuItem exception1MinMI = null;
  protected JMenuItem exception10MinMI = null;
  protected JMenuItem exception1HrMI = null;
  protected JMenuItem exception1DayMI = null;
  protected JMenuItem exceptionAllMI = null;
  protected JCheckBoxMenuItem viewHorizontalMI = null;
  protected JCheckBoxMenuItem viewVerticalMI = null;
  protected String    runningTitle = null;
  protected String    stoppedTitle = null;
  protected String    refreshMsg   = null;
  protected String    helpFile     = null;

    public MonitorFrame(String title, ReplInstanceMonitorPanel monitorPanel) {
      super(title );
      runningTitle = title;
      stoppedTitle = title + "  (Stopped)";
      this.monitorPanel = monitorPanel;
      getContentPane().setLayout( new BorderLayout() );

  	   getContentPane().add("Center", monitorPanel);

      JMenuBar menuBar = buildMenuBar();
      setJMenuBar(menuBar);
    }

	/**
    * Create the menubar on the frame.
	 * @return JMenuBar  The menubar.
	 */
    protected JMenuBar buildMenuBar() {
      JMenuBar menuBar = new JMenuBar();
      JMenu menu;

      // Build the File menu
      menu = new JMenu("File");
      menu.addMenuListener(this);
      startMI = new JMenuItem("Start");
      startMI.addActionListener( new StartAction() );
      menu.add(startMI);
      stopMI = new JMenuItem("Stop");
      stopMI.addActionListener( new StopAction() );
      menu.add(stopMI);

      // Let the panel add its own additional menu items
      Vector customMenuItemTitles = monitorPanel.getCustomMenuItemTitles();
      if (customMenuItemTitles != null && customMenuItemTitles.size() >0)  {

        Vector customMenuItemListeners = monitorPanel.getCustomMenuItemListeners();
        for (int i=0;i<customMenuItemTitles.size(); i++) {
          JMenuItem mi2 = new JMenuItem( customMenuItemTitles.elementAt(i).toString() );
          ActionListener al = (ActionListener) customMenuItemListeners.elementAt(i);
          mi2.addActionListener( al );
          menu.add(mi2);
        }
      };

      refreshMI = new JMenuItem("Change Refresh Interval");
      refreshMI.addActionListener(this);
      menu.add(refreshMI);
      exitMI = new JMenuItem("Exit");
      exitMI.addActionListener(this);
      menu.add(exitMI);
      menuBar.add(menu);
/*
      // Build the Edit menu
      menu = new JMenu("Edit");
      copyMI = new JMenuItem("Copy");
      copyMI.addActionListener(this);
      menu.add(copyMI);
      menuBar.add(menu);

      // Build the Selected menu
      menu = new JMenu("Selected");
      selectAllMI = new JMenuItem("Select All");
      selectAllMI.addActionListener(this);
      menu.add(selectAllMI);
      menuBar.add(menu);
*/
        // Build the Selected menu
      menu = new JMenu("View");
      menu.addMenuListener(this);
      if (monitorPanel.supportsMultipleGraphsInPanel() ) {
        JMenu graphLayoutMI = new JMenu("Graph Layout");

        viewHorizontalMI = new JCheckBoxMenuItem( "Horizontal", horizontalLayoutIcon, true );
        viewHorizontalMI.addActionListener(this);
        graphLayoutMI.add(viewHorizontalMI);

        viewVerticalMI = new JCheckBoxMenuItem( "Vertical", verticalLayoutIcon, false );
        viewVerticalMI.addActionListener(this);
        graphLayoutMI.add(viewVerticalMI);

        menu.add(graphLayoutMI);
      };
      JMenu traceMenu = new JMenu("Messages");

      trace1MinMI = new JMenuItem( "Last Minute" );
      trace1MinMI.addActionListener(this);
      traceMenu.add(trace1MinMI);
      trace10MinMI = new JMenuItem( "Last 10 Minutes" );
      trace10MinMI.addActionListener(this);
      traceMenu.add(trace10MinMI);
      trace1HrMI = new JMenuItem( "Last Hour" );
      trace1HrMI.addActionListener(this);
      traceMenu.add(trace1HrMI);
      trace1DayMI = new JMenuItem( "Last Day" );
      trace1DayMI.addActionListener(this);
      traceMenu.add(trace1DayMI);
      traceAllMI = new JMenuItem( "All" );
      traceAllMI.addActionListener(this);
      traceMenu.add(traceAllMI);

      menu.add(traceMenu);

      if (monitorPanel.supportExceptionTable() ) {
        JMenu exceptionMenu = new JMenu("Exceptions");

        exception1MinMI = new JMenuItem( "Last Minute" );
        exception1MinMI.addActionListener(this);
        exceptionMenu.add(exception1MinMI);
        exception10MinMI = new JMenuItem( "Last 10 Minutes" );
        exception10MinMI.addActionListener(this);
        exceptionMenu.add(exception10MinMI);
        exception1HrMI = new JMenuItem( "Last Hour" );
        exception1HrMI.addActionListener(this);
        exceptionMenu.add(exception1HrMI);
        exception1DayMI = new JMenuItem( "Last Day" );
        exception1DayMI.addActionListener(this);
        exceptionMenu.add(exception1DayMI);
        exceptionAllMI = new JMenuItem( "All" );
        exceptionAllMI.addActionListener(this);
        exceptionMenu.add(exceptionAllMI);

        menu.add(exceptionMenu);
      };

      menuBar.add(menu);

      // Build the help menu
      menu = new JMenu("Help");
      helpMI = new JMenuItem("Help...");
      helpMI.addActionListener(this);
      menu.add(helpMI);
      menu.addSeparator();
      aboutMI = new JMenuItem("About...");
      aboutMI.addActionListener(this);
      menu.add(aboutMI);
      menuBar.add(menu);

      return menuBar;
    }

	/**
    * This is called whenever a user picks a menu item.
	 * @param e   The event for the action.
	 */
    public void actionPerformed(ActionEvent e) {
      Object source = e.getSource();
      if (source == copyMI) {
        Clipboard clipboard = Toolkit.getDefaultToolkit().getSystemClipboard();
        StringSelection ss = new StringSelection("dummyString");
        clipboard.setContents(ss, ss);
      } else if (source == selectAllMI) {
        monitorPanel.selectAll();
      } else if (source == exitMI) {
        MonitorBase.closeFrameAndShutdown(this, false);   // do not call exit directly, so positions can be saved
        dispose();
      } else if (source == helpMI) {
        Utils.launchBrowser(helpFile);
      } else if (source == refreshMI) {
        JOptionPane.showMessageDialog(this, refreshMsg);
      } else if (source == trace1MinMI) {
        showTrace(60*1000);
      } else if (source == trace10MinMI) {
        showTrace(10*60*1000);
      } else if (source == trace1HrMI) {
        showTrace(60*60*1000);
      } else if (source == trace1DayMI) {
        showTrace(24*60*60*1000);
      } else if (source == traceAllMI) {
        showTrace(-1);
      } else if (source == exception1MinMI) {
        showExceptions(60*1000);
      } else if (source == exception10MinMI) {
        showExceptions(10*60*1000);
      } else if (source == exception1HrMI) {
        showExceptions(60*60*1000);
      } else if (source == exception1DayMI) {
        showExceptions(24*60*60*1000);
      } else if (source == exceptionAllMI) {
        showExceptions(-1);
      } else if (source == aboutMI) {
        String txt = "DB2 Replication Monitoring Sample V0.1\n" +
          "For quesions, contact tjacopi@us.ibm.com \n" +
          "For info on Replication, see   http://dwmaster.raleigh.ibm.com/developerworks/db2/roadmaps/sqlrepl-roadmap-v8.2.html ";
        JOptionPane.showMessageDialog(this, txt);
      } else if (source != null && source == viewHorizontalMI) {
        monitorPanel.setHorizontalLayout(true);
        pack();
      } else if (source != null && source == viewVerticalMI) {
        monitorPanel.setHorizontalLayout(false);
        pack();
      }
    }

    public void setRefreshMsg(String msg) {
      refreshMsg = msg;
    }

    public void setHelpFile(String file) {
      helpFile = file;
    }

    public ReplInstanceMonitorPanel getMonitorPanel() {
      return monitorPanel;
    }

    public void notifyRunning() {
      setTitle(runningTitle);
    }

    public void notifyStopped() {
      setTitle(stoppedTitle);
    }

    public void menuCanceled(MenuEvent e) {;}
    public void menuDeselected(MenuEvent e) {;}

    public void menuSelected(MenuEvent e) {
      boolean running = monitorPanel.isRunning();
      startMI.setEnabled(!running);
      stopMI.setEnabled(running);

      if (viewHorizontalMI != null && viewVerticalMI != null) {
        boolean isHorizontal = monitorPanel.isHorizontalLayout();
        viewHorizontalMI.setState(isHorizontal);
        viewVerticalMI.setState(!isHorizontal);
      };


    }

    public void loadInformation(Hashtable ht) {
      Rectangle rect = (Rectangle) ht.get(generateLocationSaveKey() );
      if (rect != null) {
        Dimension screenSize = Toolkit.getDefaultToolkit().getScreenSize();
        if (screenSize.width < rect.x || screenSize.height < rect.y) {       // Insure window is not off screen
          rect.x = 0;
          rect.y = 0;
        };

        setBounds(rect);
      };

      Boolean isHorizontal = (Boolean) ht.get(generateLayoutSaveKey());
      if (isHorizontal != null && !isHorizontal.booleanValue() ) {    // Horizontal is by default, so only need to set if vertical
        monitorPanel.setHorizontalLayout(false);
      };
    }

    public void saveInformation(Hashtable ht) {
      Rectangle rect = getBounds(null);
      ht.put(generateLocationSaveKey(), rect);

      boolean isHorizontal = monitorPanel.isHorizontalLayout();
      ht.put(generateLayoutSaveKey(), new Boolean(isHorizontal) );

      Vector queueNames = monitorPanel.getAllQueueNames();
      ht.put(generateQueuesSaveKey(), queueNames);
    }

    public String generateSaveKeyHeader() {
      return runningTitle;
    }

    protected String generateLocationSaveKey() {
      return generateSaveKeyHeader() + "_Position";
    }

    protected String generateLayoutSaveKey() {
      return generateSaveKeyHeader() + "_Layout";
    }

    protected String generateQueuesSaveKey() {
      return generateSaveKeyHeader() + "_Queues";
    }

    public static  String generateQueuesSaveKey(String header) {
      return header + "_Queues";
    }

   public Vector getAllQueueNames() {
      return monitorPanel.getAllQueueNames();
   }

   protected void showTrace(long fromMillis) {
     Timestamp lastTime = null;
     Connection conn = monitorPanel.getConnection();
     if (fromMillis>0) {
       String latestSQL = monitorPanel.getLatestTraceEntryQuery();
       Timestamp lastMessageTime = null;
       try {
         PreparedStatement stmt = conn.prepareStatement(latestSQL);
         ResultSet rs = stmt.executeQuery();
         while (rs.next() ) {
           lastMessageTime = rs.getTimestamp(1);
         }
         rs.close();
         stmt.close();
       } catch( Exception e ) {
         System.out.println(e);
         e.printStackTrace();
       }

       if (lastMessageTime != null) {
         lastTime = new Timestamp( lastMessageTime.getTime() - fromMillis);   // If a value was passed in, subtract it
       };
     }
     String sql = monitorPanel.getTraceQuery( (lastTime!=null) );
     Utils.showTrace(sql, lastTime, conn, monitorPanel.getTraceTitle() );
   }

   protected void showExceptions(long fromMillis) {
     Utils.showExceptions(monitorPanel.getAlias(), monitorPanel.getSchema(), monitorPanel.getConnection(), fromMillis);
   }
}