import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.util.*;
import java.sql.*;
import java.beans.*;
import java.text.*;

/**
 * This is the specific visual monitor info for QApply.
 *
 * @author tjacopi
 *
 */
public class QApplyInstanceMonitorPanel extends ReplInstanceMonitorPanel   {

    private   static final String ApplyInactiveMsg = "QApply Inactive";
    private   static final Color[] recvQLatencySegmentColors =  {Color.white, Color.green, Color.cyan };
    private   static final String[] recvQLatencySegmentText = {"{0} QApply", "{0} MQ", "{0} QCapture"};


    public QApplyInstanceMonitorPanel(String appAlias, String uid, String pwd, String appSchema) {
      super(appAlias, uid, pwd, appSchema);
    }

    protected DataGatherer createDataGatherer() {
      return new QApplyDataGatherer(schema);
    }

    protected QMonitorPanel createQMonitorPanel(String queueName) {
      QMonitorPanel monitorPanel = new QMonitorPanel(dataGatherer, queueName, recvQLatencySegmentColors, recvQLatencySegmentText);
//    monitorPanel.setLatencyNoDataMsg("Queue Stopped");
      return monitorPanel;
    }

    protected String getInactiveMsg() {
      return  ApplyInactiveMsg;
    }

    public Vector getAllQueues() {
      return getRecvQList(alias, schema, uid, pwd);
    }

    public static Vector getRecvQList(String alias, String schema, String uid, String pwd) {
      String listSQL = "SELECT DISTINCT RECVQ FROM " + schema+".IBMQREP_RECVQUEUES  ";
      return executeQueueList(listSQL, alias, schema, uid, pwd);
    }

    public boolean  supportsMultipleGraphsInPanel() {
      return true;        // may have latency and throughput graph in QMonitorPanel
    }

    protected void buildAdditionalMenuActions(Vector titles, Vector actionListeners) {
      titles.add("Add Queue");
      actionListeners.add( new AddQueueAction() );
    }

    public String getTraceQuery(boolean addParamForLastTime) {
      StringBuffer sb = new StringBuffer(100);
      sb.append("SELECT OPERATION, TRACE_TIME, DESCRIPTION FROM ");
      sb.append(schema);
      sb.append(".IBMQREP_APPLYTRACE ");
      if (addParamForLastTime) {
        sb.append(" WHERE TRACE_TIME > ?");
      };
      sb.append(" ORDER BY TRACE_TIME DESC ");

      return sb.toString();
    }

    public String getLatestTraceEntryQuery() {
      StringBuffer sb = new StringBuffer(100);
      sb.append("SELECT MAX(TRACE_TIME) FROM ");
      sb.append(schema);
      sb.append(".IBMQREP_APPLYTRACE ");

      return sb.toString();
    }

    public   String getTraceTitle() {
      return alias + "(" + schema + ")" + " QApply trace";
    }


    public boolean supportExceptionTable() {
      return true;
    }
}