import java.awt.*;
import java.awt.event.*;
import javax.swing.*;
import javax.swing.border.*;
import javax.swing.event.*;
import java.util.*;
import java.sql.*;
import java.beans.*;
import java.text.*;

/**
 * This is the panel that holds a collections of QMonitorPanels, all pertaining to one specific
 * replication instance.
 *
 * @author tjacopi
 *
 */
public abstract class ReplInstanceMonitorPanel extends JPanel implements PropertyChangeListener   {

    protected static final ImageIcon OverlayIcon = new ImageIcon("Overlay.gif");
    public    static final int LatencyGraph    = 1;
    public    static final int ThroughputGraph = 2;
    public    static final String AllQueues = "<all>";

    protected Rectangle       bounds = new Rectangle();
    protected Insets          insets = new Insets(0,0,0,0);
    protected int             noDataAvailableCount = 0;
    protected boolean         showMessageIfSlow = true;
    protected Hashtable       panels      = null;
    protected DataGatherer    dataGatherer= null;
    protected Connection      connection  = null;
    protected String          alias       = null;
    protected String          schema      = null;
    protected String          uid         = null;
    protected String          pwd         = null;
    protected boolean         horizontalLayout = true;                    // layout for graphs
    protected Vector          customMenuItemTitles = new Vector();
    protected Vector          customMenuItemListeners = new Vector();


    public ReplInstanceMonitorPanel(String alias, String uid, String pwd, String schema) {
      this.alias    = alias;
      this.schema   = schema;
      this.uid         = uid;
      this.pwd         = pwd;

      dataGatherer = createDataGatherer();
      dataGatherer.addPropertyChangeListener(this);            //Listen for when no data from QApply
      panels = new Hashtable();
      setLayout( new BorderLayout() );
      buildAdditionalMenuActions(customMenuItemTitles, customMenuItemListeners);
      addMouseListener( new ReplPanelPopupMenu() );
//    add( new JLabel(appAlias+"/"+appSchema), "West");
    }

    public boolean addMonitoring(int graphTypes, String queueName) {
      String qKey = queueName;
      if (qKey == null) {
        qKey = AllQueues;
      }

      boolean repackNecessary = false;
      boolean rebuildNecessary = false;
      QMonitorPanel qPanel = (QMonitorPanel) panels.get(qKey);
      if (qPanel == null) {
        qPanel = createQMonitorPanel(queueName);
        qPanel.setHorizontalLayout(horizontalLayout);
        panels.put(qKey, qPanel);
        rebuildNecessary = true;
        repackNecessary = true;
      }
      if (!qPanel.isShowing(graphTypes) ) {
        qPanel.show(graphTypes);
        repackNecessary = true;
      };

      if (rebuildNecessary) {
        rebuildMainPanel();
      }
      return repackNecessary;
    }

    public boolean removeMonitoring(String recvQ) {
      String recvQKey = recvQ;
      if (recvQKey == null) {
        recvQKey = AllQueues;
      }

      boolean panelRemoved = false;
      QMonitorPanel recvQPanel = (QMonitorPanel) panels.remove(recvQKey);
      if (recvQPanel != null) {
        panelRemoved = true;
      }

      if (panelRemoved) {
        rebuildMainPanel();
      }
      return panelRemoved;
    }

    public void setHorizontalLayout(boolean isHorizontal) {
      if (isHorizontal != horizontalLayout) {                    // Insure we have a change
        horizontalLayout = isHorizontal;
        for (Enumeration e = panels.elements() ; e.hasMoreElements() ;) {
          QMonitorPanel p = (QMonitorPanel) e.nextElement();
          p.setHorizontalLayout(horizontalLayout);
        }
      };
    }

    public boolean isHorizontalLayout() {
      return horizontalLayout;
    }

    protected void rebuildMainPanel() {
      removeAll();
      JPanel mainPanel = new JPanel();
      Vector panelList = getAllPanels();
      int paneCount = panelList.size();
      if (paneCount >0){
        mainPanel.setLayout( new GridLayout(paneCount,1) );
        int count = 1;

        for (int i=0; i<paneCount; i++) {
          QMonitorPanel p = (QMonitorPanel) panelList.elementAt(i);
          if ( i<(paneCount-1) ) {
            p.setSeparator( true );
          } else {
            p.setSeparator( false );
          }
          mainPanel.add( p );
        }
      /*
        for (Enumeration e = panels.elements() ; e.hasMoreElements() ;) {
          QMonitorPanel p = (QMonitorPanel) e.nextElement();
          if (count<paneCount) {
            p.setSeparator( true );
          } else {
            p.setSeparator( false );
          }
          count++;
          mainPanel.add( p );
        }
*/
      }

      add(mainPanel, "Center");
    }

   protected  Vector getAllPanels() {
     Vector panelList = new Vector();
     for (Enumeration e = panels.elements() ; e.hasMoreElements() ;) {
       QMonitorPanel p = (QMonitorPanel) e.nextElement();
       if (AllQueues.equals(p.getQueueName() ) ) {
         panelList.add(0,p);
       } else {
         panelList.addElement(p);
       }
     }
     return panelList;
   }

   public Vector getCustomMenuItemTitles() {
     return customMenuItemTitles;
   }

   public Vector getCustomMenuItemListeners() {
     return customMenuItemListeners;
   }

    public boolean isRunning() {
      return dataGatherer.isRunning();
    }

    public void start() {
      resetAllPanels();
      connection = connect(alias, uid, pwd);
      dataGatherer.start(connection);
      int delay = (dataGatherer.getTimerDelay())/1000;
      if (showMessageIfSlow && delay >15) {
        JOptionPane.showMessageDialog(this, "Note that the graph will advance after " + delay + " seconds. \nSee File->Change Refresh Interval on how to make this faster.");
      }
      showMessageIfSlow = false;
    }

    public void stop() {
      try {
        if (connection != null) {
          dataGatherer.stop();
          connection.close();
          connection = null;
        };
      } catch( Exception e ) {
        e.printStackTrace();
      }
    }

    public void resetAllPanels() {
      int paneCount = panels.size();
      if (paneCount >0){
        for (Enumeration e = panels.elements() ; e.hasMoreElements() ;) {
          QMonitorPanel p = (QMonitorPanel) e.nextElement();
          p.reset();
        }
      }
    }

    protected static Vector executeQueueList(String sql, String alias, String schema, String uid, String pwd) {
      Vector v = new Vector();
      try {
        Connection conn = connect(alias, uid, pwd);
        PreparedStatement stmt = conn.prepareStatement(sql);
        ResultSet rs = stmt.executeQuery();
        while (rs.next() ) {
          String queue = rs.getString(1);
          v.add(queue);
        }

        rs.close();
        stmt.close();
        conn.close();
      } catch( Exception e ) {
        String txt = "Error trying to fetch the list of valid queues.\nSome possible causes are:\n  1.  The schema " + schema + " is not a valid replication schema\n  2.  The userid " + uid + " does not have authority to select from the schema. \n\nJDBC Error:";

        Vector lines = MonitorBase.formatString(e.toString() );
        for (int i=0;i<lines.size();i++) {
          txt = txt + "\n" + lines.elementAt(i).toString();
        }
        e.printStackTrace();
        JOptionPane.showMessageDialog(null, txt, "Error", JOptionPane.ERROR_MESSAGE);
        System.exit(0);
      }
      return v;
    }

	/**
    * Select everything in the table.
	 */
    public void selectAll() {
      ; //do something
    }

    public QMonitorPanel getQPanelAt(int x, int y) {
      QMonitorPanel panel = null;
      Rectangle rect = new Rectangle();
      for (Enumeration e = panels.elements() ; e.hasMoreElements() && panel == null ;) {
        QMonitorPanel p = (QMonitorPanel) e.nextElement();
        p.getBounds(rect);
        if (y >= rect.y && y < (rect.y + rect.height) ) {          // Since panels span width, only need to check y coord
          panel = p;
        }
      }

      return panel;
    }

//  protected void paintComponent(Graphics g) {
//    super.paintComponent(g);

    public void paint(Graphics g) {
      super.paint(g);
      if (noDataAvailableCount > 1) {
        String inactiveMsg = getInactiveMsg();
        getBounds(bounds);
        getInsets(insets);
        bounds.x = bounds.x + insets.left;
        bounds.y = bounds.y + insets.top;
        bounds.width = bounds.width - insets.right - insets.left;
        bounds.height = bounds.height - insets.top - insets.bottom;

        // Paint the entire panel with a icon that alternates transparecy with black.
        // This achieves the "poor man" disabled look.
        if (OverlayIcon != null) {                    // Skip painting if no icon
          int iconWidth = OverlayIcon.getIconWidth();
          int iconHeight = OverlayIcon.getIconHeight();
          for (int i=0; i<bounds.width; i=i+iconWidth) {
            int amtWidth = iconWidth;
            if ((i+iconWidth)>bounds.width) {
              amtWidth= bounds.width-i;
            }
            for (int j=0; j<bounds.height; j=j+iconHeight) {
              int amtHeight = iconHeight;
              if ((j+iconHeight)>bounds.height) {
                amtHeight= bounds.height-j;
              }

              g.drawImage(OverlayIcon.getImage(), i, j, i+amtWidth, j+amtHeight,
                                              0, 0, amtWidth, amtHeight, null);
            }
          }
        }

        // Create the large font for the inactive message
        Font font = getFont();
        Font largerFont = font.deriveFont( (float) (font.getSize() + 4.0) );
        Font boldFont = largerFont.deriveFont( Font.BOLD );
        FontMetrics fm = getFontMetrics( boldFont );
        int msgWidth = fm.stringWidth(inactiveMsg);
        int msgHeight = fm.getHeight();
        int fontAscent  = fm.getAscent();
        int outsideSpacing = 10;                         // Spacing from the outside to the border
        int spacing = 10;                                // Spacing from the border to the text
        int borderSize = 3;                              // Width of border

        // First fill entire large rect with black
        int x = bounds.x + (bounds.width/2) - (msgWidth/2) - spacing - outsideSpacing;
        int y = bounds.y + (bounds.height/2) - (msgHeight/2) - spacing - outsideSpacing;
        int width = msgWidth + (2*spacing) + (2*outsideSpacing);
        int height = msgHeight + (2*spacing) + (2*outsideSpacing);
        g.setColor(Color.black);
        g.fillRect(x, y, width, height );

        // Now move in the "outsideSpacing" and fill with red.
        x = x + outsideSpacing;
        y = y + outsideSpacing;
        width = width - (2*outsideSpacing);
        height = height - (2*outsideSpacing);
        g.setColor(Color.red);
        g.fillRect(x, y, width, height );

        // Move in the border size and refill with black.  This make a red border around the black area
        x = x + borderSize;
        y = y + borderSize;
        width = width - (2*borderSize);
        height = height - (2*borderSize);
        g.setColor(Color.black);
        g.fillRect(x, y, width, height );

        // And finally draw the string
        g.setColor(Color.red);
        g.setFont(boldFont);
        g.drawString(inactiveMsg, x + (spacing-borderSize), y + spacing + fontAscent - borderSize );
        g.setFont(font);

      }
    }

    public void propertyChange(PropertyChangeEvent e) {
      if ( (DataGatherer.DataStartMsg).equals(e.getPropertyName() ) ) {
        ;   // Just ignore data start msg
      } else if ( (DataGatherer.DataEndMsg).equals(e.getPropertyName() ) ) {
        ;   // Just ignore data end msg
      } else if ( (DataGatherer.NoDataMsg).equals(e.getPropertyName() ) ) {
        noDataAvailableCount++;
        if (noDataAvailableCount == 2) {
          repaint();                // Must repaint since we are switching from "qapply active" to "inactive"
        };
      } else {
        if (noDataAvailableCount > 1) {
          repaint();                // Must repaint since we are switching from "qapply inactive" to "active"
        };
        noDataAvailableCount = 0;
      }
    }

    protected static Connection connect(String db2Database, String userid, String password) {
     Connection con = null;
    // URL is jdbc:db2:dbname  -->
     String url = "jdbc:DB2:" + db2Database;
      try {
        Class.forName("COM.ibm.db2.jdbc.app.DB2Driver");
        con = DriverManager.getConnection(url,userid,password);
      } catch( Exception e ) {
        String txt = "Error Connecting, URL used was  = " + url + "\nMost likely, your userid or password is invalid.\n\nJDBC Error:";

        Vector lines = MonitorBase.formatString(e.toString() );
        for (int i=0;i<lines.size();i++) {
          txt = txt + "\n" + lines.elementAt(i).toString();
        }
        e.printStackTrace();
        JOptionPane.showMessageDialog(null, txt, "Error", JOptionPane.ERROR_MESSAGE);
        System.exit(0);
      }
      return con;
   }

   public Vector getAllQueueNames() {
     Vector queueNames = new Vector();
      for (Enumeration e = panels.elements() ; e.hasMoreElements();) {
        QMonitorPanel p = (QMonitorPanel) e.nextElement();
        String queueName = p.getQueueName();
        if (queueName != null && queueName.length()>0) {
          queueNames.add( p.getQueueName() );
        };

      }
      return queueNames;
   }

   public Connection getConnection() {
     return connection;
   }

   public String getSchema() {
     return schema;
   }

   public String getAlias() {
     return alias;
   }

   public boolean supportExceptionTable() {
     return false;
   }

   protected abstract DataGatherer createDataGatherer();
   protected abstract QMonitorPanel createQMonitorPanel(String queueName);
   public    abstract boolean       supportsMultipleGraphsInPanel();
   protected abstract String getInactiveMsg();
   public    abstract Vector getAllQueues();
   protected abstract void buildAdditionalMenuActions(Vector titles, Vector actionListeners);
   public    abstract String getTraceQuery(boolean addParamForLastTime);
   public    abstract String getTraceTitle();
   public    abstract String getLatestTraceEntryQuery();
}